#ifndef Detector_H
#define Detector_H

#include <map>
#include <set>

#include <cstdlib>

#include <cxcparamx/simple.h>
#include <dvm3/dvm3.h>
#include <mathconst/mathconst.h>
#include <rl_basicray/rl_BasicRay.h>

#include "dbChipRow.h"
#include "Chip.h"

using namespace std;

/**
 *   The Detector class has a vector of chips, the class is responsible
 *  for reading in the chip coordinate positions.
 */
class Detector {

  friend ostream& operator << ( ostream& os, Detector& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, Detector* a ) {
    a->print( os );
    return os;
  }

public:

  typedef map< string, dbChipRow, less< string > > myMap;
  typedef myMap::const_iterator myMapIterator;
  typedef set< string, less< string > > mySet;
  typedef mySet::const_iterator DetectorSetIterator;

  Detector( cxcparamx::simple& parfile, const char* rdbheader[],
	    const char* corners_id[], const ChipSizeMap& chipSizeMap )
    ;

  void diagnostics_only( ostream& os ) const;

  double get_xaimoffset( ) { return detXY.aimoffset[0]; }
  double get_yaimoffset( ) { return detXY.aimoffset[1]; }
  double get_zaimoffset( ) { return detXY.aimoffset[2]; }

  void init_Detector( cxcparamx::simple& parfile, const char* rdbheader[],
		      const char* corners_id[],
		      const ChipSizeMap& chipSizeMap );
    

  int project_to_detector( const dvm3_Vector& dir, const dvm3_Vector& pos,
			   const MirrorPars& mirrorPars,
			   const dvm3_Vector& aimpointOSAC,
			   const dvm3_Vector& nodeOSAC,
			   DeticptResult& deticptResult, 
			   double tol=100.0*DBL_EPSILON ) const;

  void print( ostream& os ) const;

protected:

  /// The /rdb filename which contains the co-ordinate positions of the chips
  string filename;

  /// The detector is made of chips
  vector<Chip> the_chips;

  DetXY detXY;

  bool quick_check;
  bool inf_plane;

private:

  void read_database( cxcparamx::simple& parfile, const char* rdbheader[], const char* corners_id[],
		      myMap& chips, mySet& unique_chip_id );

  void check_corner_id( const string& str, const char* corners_id[] ) 
      const;

  void make_Detector( myMap& chips, const char* corners_id[],
		      const mySet& unique_chip_id,
		      const ChipSizeMap& chipSizeMap );

  double arcseconds_to_radians( double arg ) {
    return arg * M_PI / 180.0 / 3600.0;
  }

};

#endif
