#include "Photon.h"

using namespace SAOTrace::BPipeX;

namespace Deticpt {

    Photon::Photon( SAOTrace::BPipeX::Simple& bp, bool is_pixelized )
	 {

	bp.add( DpktF<DVector3>( "direction" ) );
	bp.add( DpktF<DVector3>( "position" ) );
	bp.add( DpktF<int>( "chip_id" ) );
	bp.add( DpktF<int>( "icpt" ) );

	if ( is_pixelized ) {

	    bp.add( DpktF<DVector3>( "position_np" ) );
	    bp.add( DpktF<DVector3>( "cpcpos" ) );
	    bp.add( DpktF<DVector3>( "cpcpos_np" ) );
	    bp.add( DpktF<DVector3>( "cpcdir" ) );
	    bp.add( DpktF<IVector2>( "chip" ) );
	    bp.add( DpktF<DVector2>( "chip_np" ) );
	    bp.add( DpktF<IVector2>( "tdet" ) );
	    bp.add( DpktF<DVector2>( "tdet_np" ) );
	    bp.add( DpktF<DVector2>( "det" ) );
	    bp.add( DpktF<DVector2>( "det_np" ) );

	}

	else {

	    bp.add( DpktF<DVector3>( "position_pix" ) );
	    bp.add( DpktF<DVector3>( "cpcpos_pix" ) );
	    bp.add( DpktF<DVector3>( "cpcpos" ) );
	    bp.add( DpktF<DVector3>( "cpcdir" ) );
	    bp.add( DpktF<IVector2>( "chip_pix" ) );
	    bp.add( DpktF<DVector2>( "chip" ) );
	    bp.add( DpktF<IVector2>( "tdet_pix" ) );
	    bp.add( DpktF<DVector2>( "tdet" ) );
	    bp.add( DpktF<DVector2>( "det_pix" ) );
	    bp.add( DpktF<DVector2>( "det" ) );
	}


	bp.map();
	bp.write_hdr();

	direction	= bp.dpktf_data<DVector3>( "direction" );
	position	= bp.dpktf_data<DVector3>( "position" );
	chip_id		= bp.dpktf_data<int>( "chip_id" );
	icpt		= bp.dpktf_data<int>( "icpt" );

	if ( is_pixelized ) {

	    position_np	= bp.dpktf_data<DVector3>( "position_np" );
	    cpcpos	= bp.dpktf_data<DVector3>( "cpcpos" );
	    cpcpos_np	= bp.dpktf_data<DVector3>( "cpcpos_np" );
	    cpcdir	= bp.dpktf_data<DVector3>( "cpcdir" );
	    chip	= bp.dpktf_data<IVector2>( "chip" );
	    chip_np	= bp.dpktf_data<DVector2>( "chip_np" );
	    tdet	= bp.dpktf_data<IVector2>( "tdet" );
	    tdet_np	= bp.dpktf_data<DVector2>( "tdet_np" );
	    det		= bp.dpktf_data<DVector2>( "det" );
	    det_np	= bp.dpktf_data<DVector2>( "det_np" );

	}

	else {

	    position_np	= bp.dpktf_data<DVector3>( "position_pix" );
	    cpcpos	= bp.dpktf_data<DVector3>( "cpcpos_pix" );
	    cpcpos_np	= bp.dpktf_data<DVector3>( "cpcpos" );
	    cpcdir	= bp.dpktf_data<DVector3>( "cpcdir" );
	    chip	= bp.dpktf_data<IVector2>( "chip_pix" );
	    chip_np	= bp.dpktf_data<DVector2>( "chip" );
	    tdet	= bp.dpktf_data<IVector2>( "tdet_pix" );
	    tdet_np	= bp.dpktf_data<DVector2>( "tdet" );
	    det		= bp.dpktf_data<DVector2>( "det_pix" );
	    det_np	= bp.dpktf_data<DVector2>( "det" );
	}


    }

    void Photon::set_result( DeticptResult& deticptResult ) {
	deticptResult.position		= *(position);
	deticptResult.position_np	= *(position_np);
	deticptResult.cpcpos		= *(cpcpos);
	deticptResult.cpcpos_np		= *(cpcpos_np);
	deticptResult.cpcdir		= *(cpcdir);
	deticptResult.chip		= *(chip);
	deticptResult.chip_np		= *(chip_np);
	deticptResult.tdet		= *(tdet);
	deticptResult.tdet_np		= *(tdet_np);
	deticptResult.det		= *(det);
	deticptResult.det_np		= *(det_np);
	deticptResult.chip_id		= *(chip_id);
	deticptResult.icpt		= *(icpt);
    }

    void Photon::set_photon( DeticptResult& deticptResult, const bool pixprim ) {
	*(position)    = (pixprim?deticptResult.position:deticptResult.position_np);
	*(position_np) = (pixprim?deticptResult.position_np:deticptResult.position);
	*(cpcpos)      = deticptResult.cpcpos;
	*(cpcpos_np)   = deticptResult.cpcpos_np;
	*(cpcdir)      = deticptResult.cpcdir;
	*(chip)        = deticptResult.chip;
	*(chip_np)     = deticptResult.chip_np;
	*(tdet)        = deticptResult.tdet;
	*(tdet_np)     = deticptResult.tdet_np;
	*(det)         = deticptResult.det;
	*(det_np)      = deticptResult.det_np;
	*(chip_id)     = deticptResult.chip_id;
	*(icpt)        = deticptResult.icpt;
    }

}
