#include <stdlib.h>

#include <dvm3/dvm3_vector.h>
#include <rl_basicray/rl_BasicRay.h>

#include <bpipexx/bpipexx.h>

using namespace SAOTrace;

#include <rdbxx/RDBColumn.h>
#include <rdbxx/simpleRDBTable.h>

#include "Photon.h"
#include "AimPoint.h"
#include "ChipSize.h"
#include "Detector.h"

#include "parutils.h"

void deticpt( cxcparamx::simple& parfile )
  {

  try {

    const char* rdbheader[] = { "xcpc", "ycpc", "zcpc",
				  "xcpcclip", "ycpcclip",
				  "xlsi", "ylsi", "zlsi", 
				  "chip", "corner", NULL };

    const char* corners_id[] = { "LL", "LR", "UR", "UL", NULL };

    const char* rdb_size_header[] = { "chip", "id",
                                      "xsize_pix",  "ysize_pix",
                                      "xpixelsize_micron", "ypixelsize_micron",
                                      "xsize_mm",   "ysize_mm",
				      "theta", "delta", "X0", "Y0", "H",
				      NULL };

    ChipSizeMap chipSizeMap;
    readChipSizeMapDataBase( parfile.getstr( "chipsizedb" ),
			     rdb_size_header, chipSizeMap );

    Detector detector( parfile, rdbheader, corners_id, chipSizeMap );
    //    cerr << detector << endl;
    if ( parfile.getb( "diagnostic" ) ) {
      detector.diagnostics_only( cout );
      exit( EXIT_SUCCESS );
    }

    bool pixprim = parfile.getb( "pixprim" );

    BPipeX::Simple bpipe( parfile.getstr( "input" ),
			  parfile.getstr( "output" ) );

    Deticpt::Photon photon( bpipe, pixprim );
    DVector3*  direction      = photon.direction;
    DeticptResult deticptResult;

    dvm3_Vector aimpointOSAC, nodeOSAC;
    double focal_length;

    readAimPointDataBase( parfile.getstr( "aimpointdb" ), aimpointOSAC, nodeOSAC, focal_length );

    one_of_three_ways( parfile.getstr( "xaimpoint" ),  aimpointOSAC[0] );
    one_of_three_ways( parfile.getstr( "yaimpoint" ),  aimpointOSAC[1] );
    one_of_three_ways( parfile.getstr( "zaimpoint" ),  aimpointOSAC[2] );

    // aimoffset is done in Detector since it has access to the
    // chipposdb default values...

    one_of_three_ways( parfile.getstr( "xnode" ),      nodeOSAC[0]     );
    one_of_three_ways( parfile.getstr( "ynode" ),      nodeOSAC[1]     );
    one_of_three_ways( parfile.getstr( "znode" ),      nodeOSAC[2]     );

    one_of_three_ways( parfile.getstr( "flength" ),    focal_length    );

    bool filter = parfile.getb( "filter" );

    MirrorPars mirrorPars( focal_length );

    while ( bpipe.read_dpkts( ) ) {

      photon.set_result( deticptResult );

      /* Direction and position of photon in OSAC coords */
      deticptResult.icpt = 0;

      dvm3_Vector dir( direction->x, direction->y, direction->z );
      dvm3_Vector pos( deticptResult.position.x, deticptResult.position.y, deticptResult.position.z);

      if ( EXIT_SUCCESS == detector.project_to_detector( dir, 
							 pos,
							 mirrorPars, 
							 aimpointOSAC,
							 nodeOSAC,
							 deticptResult
							 ) ) 
	deticptResult.icpt = 1;

      if ( deticptResult.icpt || filter ) {
	photon.set_photon( deticptResult, pixprim );
	bpipe.write_dpkts( );
      }

      // cout << *photon << endl;

    }

  } catch( RDBErr& re ) {
    throw( re );

  } catch ( DeticptException& de ) {
    throw( de );

  } catch ( Exception& e ) {
    throw( e );

  } catch ( ... ) {
    throw( Exception( "Unknown exception caught\n" ) );

  }

}
